package sun.management.jdp;

import java.io.IOException;
import java.util.UUID;
import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.lang.management.RuntimeMXBean;
import sun.management.VMManagement;
import java.lang.management.ManagementFactory;
import java.net.UnknownHostException;
import java.net.InetAddress;

public final class JdpController
{
    private static JdpController.JDPControllerRunner controller;
    
    private JdpController() {
    }
    
    private static int getInteger(String s, int n, String s2) throws JdpException {
        try {
            return (s == null) ? n : Integer.parseInt(s);
        }
        catch (NumberFormatException ex) {
            throw new JdpException(s2);
        }
    }
    
    private static InetAddress getInetAddress(String host, InetAddress inetAddress, String s) throws JdpException {
        try {
            return (host == null) ? inetAddress : InetAddress.getByName(host);
        }
        catch (UnknownHostException ex) {
            throw new JdpException(s);
        }
    }
    
    private static Integer getProcessId() {
        try {
            RuntimeMXBean runtimeMXBean = ManagementFactory.getRuntimeMXBean();
            Field declaredField = runtimeMXBean.getClass().getDeclaredField("jvm");
            declaredField.setAccessible(true);
            VMManagement obj = (VMManagement)declaredField.get(runtimeMXBean);
            Method declaredMethod = obj.getClass().getDeclaredMethod("getProcessId", new Class[0]);
            declaredMethod.setAccessible(true);
            return (Integer)declaredMethod.invoke(obj, new Object[0]);
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    public static synchronized void startDiscoveryService(InetAddress inetAddress, int n, String instanceName, String s) throws IOException, JdpException {
        int integer = getInteger(System.getProperty("com.sun.management.jdp.ttl"), 1, "Invalid jdp packet ttl");
        int value = getInteger(System.getProperty("com.sun.management.jdp.pause"), 5, "Invalid jdp pause") * 1000;
        InetAddress inetAddress2 = getInetAddress(System.getProperty("com.sun.management.jdp.source_addr"), null, "Invalid source address provided");
        JdpJmxPacket jdpJmxPacket = new JdpJmxPacket(UUID.randomUUID(), s);
        String property = System.getProperty("sun.java.command");
        if (property != null) {
            jdpJmxPacket.setMainClass(property.split(" ", 2)[0]);
        }
        jdpJmxPacket.setInstanceName(instanceName);
        jdpJmxPacket.setRmiHostname(System.getProperty("java.rmi.server.hostname"));
        jdpJmxPacket.setBroadcastInterval(new Integer(value).toString());
        Integer processId = getProcessId();
        if (processId != null) {
            jdpJmxPacket.setProcessId(processId.toString());
        }
        JdpBroadcaster jdpBroadcaster = new JdpBroadcaster(inetAddress, inetAddress2, n, integer);
        stopDiscoveryService();
        JdpController.controller = new JdpController.JDPControllerRunner(jdpBroadcaster, jdpJmxPacket, value);
        Thread thread = new Thread(JdpController.controller, "JDP broadcaster");
        thread.setDaemon(true);
        thread.start();
    }
    
    public static synchronized void stopDiscoveryService() {
        if (JdpController.controller != null) {
            JdpController.controller.stop();
            JdpController.controller = null;
        }
    }
    
    static {
        JdpController.controller = null;
    }
    
    private static class JDPControllerRunner implements Runnable
    {
        private final JdpJmxPacket packet;
        private final JdpBroadcaster bcast;
        private final int pause;
        private volatile boolean shutdown;
        
        private JDPControllerRunner(JdpBroadcaster bcast, JdpJmxPacket packet, int pause) {
            this.shutdown = false;
            this.bcast = bcast;
            this.packet = packet;
            this.pause = pause;
        }
        
        @Override
        public void run() {
            try {
                while (!this.shutdown) {
                    this.bcast.sendPacket(this.packet);
                    try {
                        Thread.sleep((long)this.pause);
                    }
                    catch (InterruptedException ex) {}
                }
            }
            catch (IOException ex2) {}
            try {
                this.stop();
                this.bcast.shutdown();
            }
            catch (IOException ex3) {}
        }
        
        public void stop() {
            this.shutdown = true;
        }
    }
}
